package com.ubsidi.menu;

import android.app.Activity;
import android.app.job.JobInfo;
import android.app.job.JobScheduler;
import android.content.BroadcastReceiver;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.media.MediaPlayer;
import android.os.Build;
import android.provider.Settings;

import androidx.appcompat.app.AppCompatDelegate;
import androidx.localbroadcastmanager.content.LocalBroadcastManager;
import androidx.multidex.MultiDexApplication;
import androidx.room.Room;

import com.androidnetworking.AndroidNetworking;
//import com.google.firebase.crashlytics.FirebaseCrashlytics;
//import com.google.firebase.messaging.FirebaseMessaging;
import com.google.gson.Gson;
import com.ubsidi.menu.activities.NoInternetActivity;
import com.ubsidi.menu.activities.SplashActivity;
import com.ubsidi.menu.models.Counts;
import com.ubsidi.menu.models.MerchantApp;
import com.ubsidi.menu.models.MerchantServiceDetails;
import com.ubsidi.menu.models.MerchantSupervisor;
import com.ubsidi.menu.models.RequestResponse;
import com.ubsidi.menu.models.SiteSetting;
import com.ubsidi.menu.network.NetworkInterceptor;
import com.ubsidi.menu.services_receivers.TiffintomPartnerJobService;
import com.ubsidi.menu.services_receivers.TiffintomPartnerService;
import com.ubsidi.menu.storageutils.AppDatabase;
import com.ubsidi.menu.storageutils.MyPreferences;
import com.ubsidi.menu.utils.Constants;
import com.ubsidi.menu.utils.LogUtils;

import org.json.JSONObject;

import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.concurrent.TimeUnit;

import okhttp3.OkHttpClient;

/**
 * Created by Amrish on 07-11-2020.
 */
public class MyApp extends MultiDexApplication {
    public static MyApp instance;
    private MyPreferences myPreferences;

    public static MyApp getInstance() {
        return instance;
    }

    public String currencySymbol = "£";

    public String getCurrencySymbol() {
        return currencySymbol;
    }

    public MyPreferences getMyPreferences() {
        return myPreferences;
    }

    public AppDatabase appDatabase;
    public static String DEL_TIMEFORMAT = "yyyy-MM-dd";
    public static String PHP_TIMEFORMAT = "yyyy-MM-dd HH:mm:ss";
    public static String PHP_DATE_TIME_FORMAT_ZULU = "yyyy-MM-dd'T'HH:mm:ss.SSSSSS'Z'";
    private MediaPlayer mp, delayedOrderMP, paymentSuccessMP, paymentFailedMP, newMessageMP, mpBookings;
    private BroadcastReceiver broadcastReceiver;
    public Counts allCounts;
    public Bitmap merchantLogo, restaurantLogo;
    public ArrayList<SiteSetting> siteSettings = new ArrayList<>();

    public MerchantApp getMerchantApp() {
        if (getMyPreferences().getLoggedInMerchant() != null) {
            return getMyPreferences().getMerchantApp();
        }
        return null;
    }

    public MerchantServiceDetails getMerchantServiceDetails() {
        MerchantApp merchantApp = getMerchantApp();
        if (merchantApp != null && merchantApp.merchantService != null && merchantApp.merchantService.size() > 0) {
            return merchantApp.merchantService.get(0);
        } else return null;

    }

    public MerchantSupervisor getSupervisorDetails() {
        MerchantApp merchantApp = getMerchantApp();
        if (merchantApp != null && merchantApp.supervisor_info != null && merchantApp.supervisor_info.size() > 0) {
            return merchantApp.supervisor_info.get(0);
        } else return null;

    }

    @Override
    public void onCreate() {
        super.onCreate();
        AppCompatDelegate.setDefaultNightMode(AppCompatDelegate.MODE_NIGHT_NO);
        instance = this;
        myPreferences = new MyPreferences(this);
//        initOrderSoundMP();
        initPaymentFailedMP();
        initPaymentSuccessMP();
        initDelayedOrderSoundMP();
        initNewMessageSoundMP();
//        initBookingSoundMP();
        getFirebaseToken();
//        FirebaseCrashlytics.getInstance().setCrashlyticsCollectionEnabled(true);
        getMyPreferences().setAndroidDeviceId(Settings.Secure.getString(getContentResolver(), Settings.Secure.ANDROID_ID));
        appDatabase = Room.databaseBuilder(getApplicationContext(),
                AppDatabase.class, "tiffintom_partner-database")
                .fallbackToDestructiveMigration()
                .allowMainThreadQueries()
                .build();
        new Thread(this::initializeApi).start();

        broadcastReceiver = new BroadcastReceiver() {
            @Override
            public void onReceive(Context context, Intent intent) {
                if (intent.getBooleanExtra("refresh", false)) {
                    allCounts = new Gson().fromJson(intent.getStringExtra("counts"), Counts.class);
                    if (allCounts != null && allCounts.pending_orders > 0) {
                        startOrderSound();
                    } else {
                        stopOrderSound();
                    }
                    if (allCounts != null && allCounts.delay_orders > 0) {
                        startDelayedOrderSound();
                    } else {
                        stopDelayedOrderSound();
                    }
                    if (allCounts != null && allCounts.unread_chat > 0) {
                        startNewMessageSound();
                    } else {
                        stopNewMessageSound();
                    }
                    if (allCounts != null && allCounts.pending_bookings > 0) {
                        startBookingSound();
                    } else {
                        stopBookingSound();
                    }
                }
            }
        };

        LocalBroadcastManager.getInstance(this).registerReceiver(broadcastReceiver, new IntentFilter(Constants.NOTIFICATION));

    }

    private void getFirebaseToken() {
       /* FirebaseMessaging.getInstance().getToken().addOnCompleteListener(task -> {
            if (!task.isSuccessful()) {
                return;
            }
            String token = task.getResult();
            myPreferences.setUserFCMToken(token);
        });*/
    }

    public void generateMerchantLogoBitmap() {
        new Thread(() -> {
            try {
                URL url = new URL(getMyPreferences().getMerchantBusinesses().image_url);
                merchantLogo = BitmapFactory.decodeStream(url.openConnection().getInputStream());
                LogUtils.e("Merchant logo downloaded");
            } catch (IOException e) {
                e.printStackTrace();
            }
        }).start();

    }

    public void generateRestaurantLogoBitmap() {
        new Thread(() -> {
//            restaurantLogo = BitmapFactory.decodeResource(this.getResources(), R.drawable.printer_tiffinotom_logo);
        }).start();

    }

    public void initService() {
        try {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                schedulerJob(getApplicationContext());
            } else {
                startService(new Intent(getApplicationContext(), TiffintomPartnerService.class));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void schedulerJob(Context context) {
        try {
            ComponentName serviceComponent = new ComponentName(context, TiffintomPartnerJobService.class);
            JobInfo.Builder builder = new JobInfo.Builder(0, serviceComponent);
            builder.setMinimumLatency(5000);    // wait at least
            builder.setOverrideDeadline(30000);  //delay time
            builder.setRequiredNetworkType(JobInfo.NETWORK_TYPE_ANY);  // require unmetered network
            builder.setRequiresCharging(false);  // we don't care if the device is charging or not
            builder.setRequiresDeviceIdle(false); // device should be idle
            System.out.println("(scheduler Job");
            JobScheduler jobScheduler = null;
            if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.M) {
                jobScheduler = context.getSystemService(JobScheduler.class);
            }
            jobScheduler.schedule(builder.build());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }


    private void initializeApi() {
        OkHttpClient okHttpClient = new OkHttpClient().newBuilder()
                .connectTimeout(1, TimeUnit.MINUTES)
                .readTimeout(1, TimeUnit.MINUTES)
                .writeTimeout(1, TimeUnit.MINUTES)
                .addInterceptor(new NetworkInterceptor())
                .build();
        // .addInterceptor(new ChuckerInterceptor(getApplicationContext()))
        AndroidNetworking.initialize(getApplicationContext(), okHttpClient);
    }

    public String checkErrorMessage(JSONObject jsonObject) {
        RequestResponse requestResponse = new Gson().fromJson(jsonObject.toString(), RequestResponse.class);
        if (requestResponse.result != null) {
            if (requestResponse.result.success != 1) {
                if (requestResponse.result.message != null)
                    return requestResponse.result.message;
                else return "No results found";
            } else {
                return null;
            }
        } else {
            return "No results found";
        }
    }


    public String getStripePublishKey() {
        if (getMyPreferences().getSiteSettings().stripe_mode.equalsIgnoreCase("test")) {
            return getMyPreferences().getSiteSettings().stripe_publishkeyTest;
        } else {
            return getMyPreferences().getSiteSettings().stripe_publishkey;
        }
    }

    public boolean isLoggedIn() {
        return getMyPreferences().getLoggedInRestaurant() != null;
    }

    public boolean isMerchantLogin() {
        return getMyPreferences().getLoggedInMerchant() != null;
    }

    public void logout(Activity activity) {
       /* if (getMyPreferences().getLoggedInRestaurant() != null) {
            AndroidNetworking.post(ApiEndPoints.BASE_URL()).addBodyParameter("action", "StoreLogout").addBodyParameter("user_id", getMyPreferences().getLoggedInRestaurant().id).build().getAsJSONObject(new JSONObjectRequestListener() {
                @Override
                public void onResponse(JSONObject response) {
                    LogUtils.e("Logout response: ", new Gson().toJson(response));
                }

                @Override
                public void onError(ANError anError) {
                    anError.printStackTrace();
                    LogUtils.e("Logout error: ");
                }
            });
        }*/
        getMyPreferences().logout();
        if (getMyPreferences().getLoggedInMerchant() == null) {
            getMyPreferences().saveDefaultPrinter(null);
            getMyPreferences().saveAppIntroDone(false);
        }
        startActivity(new Intent(activity, SplashActivity.class).addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
    }

    public void restaurantLogout(Activity activity, boolean shouldRedirect) {
        /*if (getMyPreferences().getLoggedInRestaurant() != null) {
            AndroidNetworking.post(ApiEndPoints.BASE_URL()).addBodyParameter("action", "StoreLogout").addBodyParameter("user_id", getMyPreferences().getLoggedInRestaurant().id).build().getAsJSONObject(new JSONObjectRequestListener() {
                @Override
                public void onResponse(JSONObject response) {
                    LogUtils.e("Logout response: ", new Gson().toJson(response));
                }

                @Override
                public void onError(ANError anError) {
                    anError.printStackTrace();
                    LogUtils.e("Logout error: ");
                }
            });
        }*/
        getMyPreferences().logout();
        if (getMyPreferences().getLoggedInMerchant() == null) {
            getMyPreferences().saveDefaultPrinter(null);
            getMyPreferences().saveAppIntroDone(false);
        }
        if (shouldRedirect)
            startActivity(new Intent(activity, SplashActivity.class).addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
    }

    public void logoutMerchant(Activity activity) {
        getMyPreferences().logoutMerchant();
        if (getMyPreferences().getLoggedInRestaurant() == null) {
            getMyPreferences().saveDefaultPrinter(null);
        }
        startActivity(new Intent(activity, SplashActivity.class).addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
    }

    public void logoutMerchant(Activity activity, boolean shouldRedirect) {
        getMyPreferences().logoutMerchant();
        if (getMyPreferences().getLoggedInRestaurant() == null) {
            getMyPreferences().saveDefaultPrinter(null);
        }
        if (shouldRedirect)
            startActivity(new Intent(activity, SplashActivity.class).addFlags(Intent.FLAG_ACTIVITY_NEW_TASK | Intent.FLAG_ACTIVITY_CLEAR_TASK | Intent.FLAG_ACTIVITY_CLEAR_TOP));
    }

    public void noInternet(Activity activity) {
        if (activity != null)
            startActivity(new Intent(activity, NoInternetActivity.class));
    }

    public static class df {
        public static String format(float value) {
            return String.format("%.2f", value);
        }
    }

    public ArrayList<String> getPrinters() {
        ArrayList<String> printers = new ArrayList<>();
        printers.add("V1");
        printers.add("V1S");
        printers.add("V2");
        printers.add("Zonerich");
        printers.add("Mobile");
        printers.add("KP-80");
        return printers;
    }

    public void startOrderSound() {
        if (mp != null) {
            mp.start();
            mp.setLooping(true);
            mp.setVolume(100, 100);
        }
    }

    public void stopOrderSound() {
        if (mp != null && mp.isPlaying()) {
            mp.stop();
            mp.reset();
            mp.release();
//            initOrderSoundMP();
        }
    }

    public void startBookingSound() {
        if (mpBookings != null) {
            mpBookings.start();
            mpBookings.setLooping(true);
            mpBookings.setVolume(100, 100);
        }
    }

    public void stopBookingSound() {
        if (mpBookings != null && mpBookings.isPlaying()) {
            mpBookings.stop();
            mpBookings.reset();
            mpBookings.release();
//            initBookingSoundMP();
        }
    }


    public void startDelayedOrderSound() {
        if (delayedOrderMP != null) {
            delayedOrderMP.start();
            delayedOrderMP.setLooping(true);
            delayedOrderMP.setVolume(100, 100);
        }
    }

    public void stopDelayedOrderSound() {
        if (delayedOrderMP != null && delayedOrderMP.isPlaying()) {
            delayedOrderMP.stop();
            delayedOrderMP.reset();
            delayedOrderMP.release();
            initDelayedOrderSoundMP();
        }
    }

//    private void initOrderSoundMP() {
//        mp = MediaPlayer.create(this, R.raw.notification);
//    }

//    private void initBookingSoundMP() {
//        mpBookings = MediaPlayer.create(this, R.raw.notification);
//    }

    private void initDelayedOrderSoundMP() {
//        delayedOrderMP = MediaPlayer.create(this, R.raw.delayed_order);
    }

    private void initPaymentSuccessMP() {
//        paymentSuccessMP = MediaPlayer.create(this, R.raw.payment_approval_sound);
    }

    private void initNewMessageSoundMP() {
//        newMessageMP = MediaPlayer.create(this, R.raw.new_message);
    }

    public void startPaymentSuccessSound() {
        if (paymentSuccessMP != null) {
            paymentSuccessMP.start();
            paymentSuccessMP.setVolume(100, 100);
        }
    }

    public void stopPaymentSuccessSound() {
        if (paymentSuccessMP != null && paymentSuccessMP.isPlaying()) {
            paymentSuccessMP.stop();
            paymentSuccessMP.reset();
            paymentSuccessMP.release();
            initPaymentSuccessMP();
        }
    }

    public void startNewMessageSound() {
        if (newMessageMP != null) {
            newMessageMP.start();
            newMessageMP.setVolume(100, 100);
        }
    }

    public void stopNewMessageSound() {
        if (newMessageMP != null && newMessageMP.isPlaying()) {
            newMessageMP.stop();
            newMessageMP.reset();
            newMessageMP.release();
            initNewMessageSoundMP();
        }
    }

    private void initPaymentFailedMP() {
//        paymentFailedMP = MediaPlayer.create(this, R.raw.decline_sound);
    }

    public void startPaymentFailedSound() {
        if (paymentFailedMP != null) {
            paymentFailedMP.start();
            paymentFailedMP.setVolume(100, 100);
        }
    }

    public void stopPaymentFailedSound() {
        if (paymentFailedMP != null && paymentFailedMP.isPlaying()) {
            paymentFailedMP.stop();
            paymentFailedMP.reset();
            paymentFailedMP.release();
            initPaymentFailedMP();
        }
    }


}
