package com.ubsidi.menu.activities;

import androidx.appcompat.app.AppCompatActivity;
import androidx.fragment.app.Fragment;
import androidx.fragment.app.FragmentManager;
import androidx.fragment.app.FragmentTransaction;

import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.LinearLayout;
import android.widget.RelativeLayout;
import android.widget.TextView;

import com.ubsidi.menu.R;
import com.ubsidi.menu.base.BaseActivity;
import com.ubsidi.menu.fragments.ConfirmationExitDialogFragment;
import com.ubsidi.menu.fragments.MenuFragment;
import com.ubsidi.menu.interfaces.DialogDismissListener;
import com.ubsidi.menu.utils.LogUtils;

import java.util.Objects;

public class HomeActivity extends BaseActivity {

    LinearLayout ll_home;
    TextView tvRestaurantName, tvChangeRestaurant;
    RelativeLayout ll_Menu;
    TextView tvMenuRestaurantName, tvService;
    LinearLayout llEpos, llOnline;
    FrameLayout hostFragment;

    @Override
    protected void initViews() {
        super.initViews();

        ll_home = findViewById(R.id.ll_home);
        tvRestaurantName = findViewById(R.id.tvRestaurantName);
        tvRestaurantName.setText(myApp.getMyPreferences().getLoggedInRestaurant().restaurant_name);
        tvChangeRestaurant = findViewById(R.id.tvChangeRestaurant);

        ll_Menu = findViewById(R.id.ll_Menu);
        tvMenuRestaurantName = findViewById(R.id.tvMenuRestaurantName);
        tvMenuRestaurantName.setText(myApp.getMyPreferences().getLoggedInRestaurant().restaurant_name);
        tvService = findViewById(R.id.tvService);

        llEpos = findViewById(R.id.ll_epos);
        llOnline = findViewById(R.id.ll_online);

        hostFragment = findViewById(R.id.nav_host_fragment);
        setService();
    }

    @Override
    protected void setXML() {
        super.setXML();
        setContentView(R.layout.activity_home);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        tvChangeRestaurant.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                Bundle bundle = new Bundle();
                bundle.putString("destination", "restaurant_selection");
                bundle.putBoolean("finish", true);
                new Handler(Objects.requireNonNull(Looper.myLooper()))
                        .postDelayed(new Runnable() {
                            @Override
                            public void run() {
                                startActivity(new Intent(HomeActivity.this, TransportActivity.class).putExtras(bundle));
                            }
                        }, 1500);
            }
        });

        llEpos.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                ll_home.setVisibility(View.GONE);
                ll_Menu.setVisibility(View.VISIBLE);
                tvService.setText("Epos");
                llEpos.setVisibility(View.GONE);
                llOnline.setVisibility(View.GONE);
                hostFragment.setVisibility(View.VISIBLE);

                FragmentManager manager = getSupportFragmentManager();
                FragmentTransaction transaction = manager.beginTransaction();
                transaction.replace(R.id.nav_host_fragment, MenuFragment.getInstance("epos"));
                transaction.addToBackStack("can_go_back");
                transaction.commit();
            }
        });

        llOnline.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                ll_home.setVisibility(View.GONE);
                ll_Menu.setVisibility(View.VISIBLE);
                tvService.setText("Online");
                llEpos.setVisibility(View.GONE);
                llOnline.setVisibility(View.GONE);
                hostFragment.setVisibility(View.VISIBLE);

                FragmentManager manager = getSupportFragmentManager();
                FragmentTransaction transaction = manager.beginTransaction();
                transaction.replace(R.id.nav_host_fragment, MenuFragment.getInstance("epos"));
                transaction.addToBackStack("can_go_back");
                transaction.commit();
            }
        });
    }

    private void setService() {
        if (myApp.getMyPreferences().getMerchantBusinesses().epos_service)
            llEpos.setVisibility(View.VISIBLE);

        if (myApp.getMyPreferences().getMerchantBusinesses().online_service)
            llOnline.setVisibility(View.VISIBLE);
    }

    @Override
    public void onBackPressed() {
        try {
            Fragment openedFragment = getSupportFragmentManager().findFragmentById(R.id.nav_host_fragment);
            FragmentManager manager = getSupportFragmentManager();

            if (hostFragment.getVisibility() == View.GONE) {
                askExitConfirmation();
            } else if (openedFragment instanceof MenuFragment) {
                ll_home.setVisibility(View.VISIBLE);
                ll_Menu.setVisibility(View.GONE);
                setService();
                hostFragment.setVisibility(View.GONE);
            } else {
                if (manager.getBackStackEntryCount() > 0) {
                    manager.popBackStack();
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void askExitConfirmation() {
        try {
            ConfirmationExitDialogFragment confirmationExitDialogFragment = ConfirmationExitDialogFragment.getInstance();
            confirmationExitDialogFragment.show(getSupportFragmentManager(), "exit");
            confirmationExitDialogFragment.setDialogDismissListener(new DialogDismissListener() {
                @Override
                public void onDialogDismiss(Object o) {
                    if (o instanceof String) {
                        if (((String) o).equalsIgnoreCase("confirm")) {
                            finishAffinity();
                        }
                    }
                }
            });

//            confirmationExitDialogFragment.setDialogDismissListener(o -> {
//                if (o instanceof String) {
//                    if (((String) o).equalsIgnoreCase("confirm")) {
//                        finishAffinity();
//                    }
//                }
//            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}