package com.ubsidi.menu.adapters;

import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;
import androidx.recyclerview.widget.RecyclerView;

import com.ubsidi.menu.R;
import com.ubsidi.menu.interfaces.RecyclerViewItemClickListener;
import com.ubsidi.menu.models.Addon;
import com.ubsidi.menu.models.Categories;
import com.ubsidi.menu.models.Product;

import java.util.ArrayList;

public class CategoryAdapter extends RecyclerView.Adapter<CategoryAdapter.ViewHolder> {

    private ArrayList<Object> feed;
    private RecyclerViewItemClickListener itemViewClickListener;

    public CategoryAdapter(ArrayList<Object> categories, RecyclerViewItemClickListener itemViewClickListener) {
        this.feed = categories;
        this.itemViewClickListener = itemViewClickListener;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.item_menu_items_list, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        if (feed.get(position) instanceof Categories) {
            Categories category = (Categories) feed.get(position);
            holder.tvCategoryName.setText(category.category_name);

            Context context = holder.itemView.getContext();
            if (category.status.equalsIgnoreCase("true")) {
                holder.tvStatus.setText("Active");
                holder.tvStatus.setTextColor(ContextCompat.getColor(context, R.color.chip_green_color));
            } else {
                holder.tvStatus.setText("Inactive");
                holder.tvStatus.setTextColor(ContextCompat.getColor(context, R.color.reject_color));

            }

            holder.tvEditDelete.setOnClickListener(view -> {
                if (itemViewClickListener != null) {
                    itemViewClickListener.onItemClick(position, category);
                }
            });
        } else if (feed.get(position) instanceof Addon) {
            Addon addon = (Addon) feed.get(position);
            holder.tvCategoryName.setText(addon.mainaddons_name);
            holder.tvSubName.setVisibility(View.VISIBLE);
            holder.tvSubName.setText(addon.category_name);

            Context context = holder.itemView.getContext();
            if (addon.status.equalsIgnoreCase("true")) {
                holder.tvStatus.setText("Active");
                holder.tvStatus.setTextColor(ContextCompat.getColor(context, R.color.chip_green_color));
            } else {
                holder.tvStatus.setText("Inactive");
                holder.tvStatus.setTextColor(ContextCompat.getColor(context, R.color.reject_color));

            }

            holder.tvEditDelete.setOnClickListener(view -> {
                if (itemViewClickListener != null) {
                    itemViewClickListener.onItemClick(position, addon);
                }
            });
        } else if (feed.get(position) instanceof Product) {
            Product addon = (Product) feed.get(position);
            holder.tvCategoryName.setText(addon.menu_name);
            holder.tvSubName.setVisibility(View.VISIBLE);
            holder.tvSubName.setText(addon.category_name);

            Context context = holder.itemView.getContext();
            if (addon.status.equalsIgnoreCase("true")) {
                holder.tvStatus.setText("Active");
                holder.tvStatus.setTextColor(ContextCompat.getColor(context, R.color.chip_green_color));
            } else {
                holder.tvStatus.setText("Inactive");
                holder.tvStatus.setTextColor(ContextCompat.getColor(context, R.color.reject_color));

            }
            holder.tvEditDelete.setOnClickListener(view -> {
                if (itemViewClickListener != null) {
                    itemViewClickListener.onItemClick(position, addon);
                }
            });
        }

    }

    @Override
    public int getItemCount() {
        return feed.size();
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        TextView tvSubName, tvCategoryName, tvEditDelete, tvStatus;

        public ViewHolder(@NonNull View itemView) {
            super(itemView);
            tvSubName = itemView.findViewById(R.id.tvSubName);
            tvCategoryName = itemView.findViewById(R.id.tvCategoryName);
            tvEditDelete = itemView.findViewById(R.id.tvEditDelete);
            tvStatus = itemView.findViewById(R.id.tvStatus);
        }
    }
}
