package com.ubsidi.menu.adapters;

import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.widget.AppCompatEditText;
import androidx.recyclerview.widget.RecyclerView;

import com.google.android.material.button.MaterialButton;
import com.ubsidi.menu.R;
import com.ubsidi.menu.common.Validators;
import com.ubsidi.menu.interfaces.MoneyTextWatcher;
import com.ubsidi.menu.interfaces.RecyclerViewItemClickListener;
import com.ubsidi.menu.models.Header;

import java.util.ArrayList;

public class SubAddonPriceAdapter extends RecyclerView.Adapter<SubAddonPriceAdapter.ViewHolder> {
    private ArrayList<Header> priceList;
    private RecyclerViewItemClickListener itemViewClickListener;

    public SubAddonPriceAdapter(ArrayList<Header> priceList, RecyclerViewItemClickListener itemViewClickListener) {
        this.priceList = priceList;
        this.itemViewClickListener = itemViewClickListener;
    }

    @NonNull
    @Override
    public ViewHolder onCreateViewHolder(@NonNull ViewGroup parent, int viewType) {
        return new ViewHolder(LayoutInflater.from(parent.getContext()).inflate(R.layout.item_sub_addon_price, parent, false));
    }

    @Override
    public void onBindViewHolder(@NonNull ViewHolder holder, int position) {
        Header header = priceList.get(position);
        holder.tvPrice.setText(header.total);

        holder.btnAdd.setOnClickListener(view -> {
            if (!Validators.isNullOrEmpty(holder.etPrice.getText().toString())) {
                holder.etPrice.setError(null);
                header.title = holder.etPrice.getText().toString();
                if (itemViewClickListener != null) {
                    itemViewClickListener.onItemClick(position, header);
                }
            } else {
                holder.etPrice.setError("Please enter price");
            }

        });

    }

    @Override
    public int getItemCount() {
        return priceList.size();
    }

    public class ViewHolder extends RecyclerView.ViewHolder {
        TextView tvPrice;
        AppCompatEditText etPrice;
        MaterialButton btnAdd;

        public ViewHolder(@NonNull View itemView) {
            super(itemView);
            tvPrice = itemView.findViewById(R.id.tvPrice);
            etPrice = itemView.findViewById(R.id.etPrice);
            btnAdd = itemView.findViewById(R.id.btnAdd);
            etPrice.addTextChangedListener(new MoneyTextWatcher(etPrice));
        }
    }
}
