package com.ubsidi.menu.fragments;

import static com.ubsidi.menu.utils.Constants.CODE_IMAGE_PICKER;

import android.content.Intent;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatEditText;
import androidx.appcompat.widget.SwitchCompat;
import androidx.core.content.ContextCompat;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.GridLayoutManager;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.LinearLayout;
import android.widget.RadioButton;
import android.widget.TextView;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.github.dhaval2404.imagepicker.ImagePicker;
import com.google.android.material.button.MaterialButton;
import com.google.android.material.card.MaterialCardView;
import com.google.android.material.textfield.MaterialAutoCompleteTextView;
import com.google.gson.Gson;
import com.makeramen.roundedimageview.RoundedImageView;
import com.ubsidi.menu.R;
import com.ubsidi.menu.adapters.AddPriceVariantDialogFragment;
import com.ubsidi.menu.adapters.AllergiesitemAdapter;
import com.ubsidi.menu.adapters.CategoryAutoSuggestedAdapter;
import com.ubsidi.menu.adapters.PriceVarientListAdapter;
import com.ubsidi.menu.adapters.SubAddonListAdapter;
import com.ubsidi.menu.base.BaseBottomSheet;
import com.ubsidi.menu.common.CommonFunctions;
import com.ubsidi.menu.common.Validators;
import com.ubsidi.menu.interfaces.DialogDismissListener;
import com.ubsidi.menu.interfaces.MoneyTextWatcher;
import com.ubsidi.menu.interfaces.RecyclerViewItemClickListener;
import com.ubsidi.menu.models.AddAddon;
import com.ubsidi.menu.models.AddMainAddon;
import com.ubsidi.menu.models.AddSubAddon;
import com.ubsidi.menu.models.Allergens;
import com.ubsidi.menu.models.Categories;
import com.ubsidi.menu.models.Product;
import com.ubsidi.menu.models.SubAddon;
import com.ubsidi.menu.models.Variants;
import com.ubsidi.menu.network.ApiEndPoints;
import com.ubsidi.menu.utils.LogUtils;
import com.ubsidi.menu.utils.ToastUtils;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import worker8.com.github.radiogroupplus.RadioGroupPlus;

public class AddProductBottomSheetFragment extends BaseBottomSheet {

    private MaterialCardView cvNoImage;
    private RoundedImageView ivImage;
    private AppCompatEditText etProduct, etShortName, etProductDesc, etSortOrder, etProductDiscount, etProductPrice;
    private TextView tvSwitchStatus, tvAddImage;
    private SwitchCompat switchOrder;
    private RadioGroupPlus rbGroup, rbAddonGroup;
    private RadioButton rbSingle, rbMultiple, rbYes, rbNo;
    private MaterialButton btnCancel, btnConfirm;
    private MaterialAutoCompleteTextView etCategory;
    private LinearLayout llSingleLayout;
    private LinearLayout llAddPriceVariant, llAddSubAddon, llMultipleLayout, llSubAddonLayout;
    private RecyclerView rvPriceVariant, rvAllergies, rvSubAddon;
    private DialogDismissListener dialogDismissListener;
    private int selectedCategoryId = -1;
    public static int selectedAllergiesCount = 0;

    private ArrayList<Categories> categories = new ArrayList<>();
    private ArrayAdapter<Categories> autoadapter;
    private ArrayList<Variants> priceVariants = new ArrayList<>();
    private ArrayList<Variants> variantsArrayList = new ArrayList<>();
    private ArrayList<Allergens> allergensArrayList = new ArrayList<>();
    private ArrayList<SubAddon> selectedAddons = new ArrayList<>();
    private ArrayList<AddMainAddon> addMainAddons = new ArrayList<>();

    private Allergens allergens;
    private PriceVarientListAdapter priceVarientAdapter;
    private SubAddonListAdapter subAddonAdapter;
    private AllergiesitemAdapter allergiesitemAdapter;

    public static AddProductBottomSheetFragment getInstance(String subjectData) {
        AddProductBottomSheetFragment feedbackSubjectBottomSheetFragment = new AddProductBottomSheetFragment();
        Bundle bundle = new Bundle();
        bundle.putString("selected_subject", subjectData);
        feedbackSubjectBottomSheetFragment.setArguments(bundle);
        return feedbackSubjectBottomSheetFragment;
    }

    public void setDialogDismissListener(DialogDismissListener dialogDismissListener) {
        this.dialogDismissListener = dialogDismissListener;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        fetchCategory();
        return inflater.inflate(R.layout.fragment_add_product_bottom_sheet, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

    }

    @Override
    protected void manageIntents() {
        super.manageIntents();

    }

    @Override
    protected void setListeners() {
        super.setListeners();
        tvAddImage.setOnClickListener(view -> {
            pickImage();
        });

        etCategory.setOnKeyListener((textView, i, keyEvent) -> {
            if (keyEvent.getAction() != KeyEvent.ACTION_DOWN)
                return true;
            if (keyEvent.getKeyCode() == KeyEvent.KEYCODE_BACK) {
                getActivity().onBackPressed();
            } else if ((keyEvent.getKeyCode() == KeyEvent.KEYCODE_ENTER)) {
                CommonFunctions.hideKeyboard(getActivity(), etCategory);
            }
            return false;
        });
        switchOrder.setOnCheckedChangeListener((compoundButton, b) -> {
            if (b) {
                tvSwitchStatus.setText("Active");
                tvSwitchStatus.setTextColor(ContextCompat.getColor(getActivity(), R.color.chip_green_color));
            } else {
                tvSwitchStatus.setText("Inactive");
                tvSwitchStatus.setTextColor(ContextCompat.getColor(getActivity(), R.color.reject_color));
            }
        });

        llAddPriceVariant.setOnClickListener(view -> {
            AddPriceVariantDialogFragment dialogFragment = AddPriceVariantDialogFragment.getInstance(null, 0, true);
            dialogFragment.show(getChildFragmentManager(), "add price variant");
            dialogFragment.setDialogDismissListener(new DialogDismissListener() {
                @Override
                public void onDialogDismiss(Object o) {
                    Variants variants = (Variants) o;
                    LogUtils.e("Price Variant Data::" + new Gson().toJson(variants));
                    priceVariants.add(variants);
                    priceVarientAdapter.notifyDataSetChanged();
                }
            });

//            dialogFragment.setDialogDismissListener(o -> {
//                Variants variants = (Variants) o;
//                LogUtils.e("Price Variant Data::" + new Gson().toJson(variants));
//                priceVariants.add(variants);
//                priceVarientAdapter.notifyDataSetChanged();
//            });
        });

        llAddSubAddon.setOnClickListener(view -> {
            if (selectedCategoryId != -1) {
                if (isSelectedItem()) {
                    if (rbSingle.isChecked()) {
                        variantsArrayList.clear();
                        Variants vr = new Variants();
                        vr.sub_name = etProduct.getText().toString();
                        vr.orginal_price = etProductPrice.getText().toString();
                        variantsArrayList.add(vr);
                    } else if (rbMultiple.isChecked()) {
                        variantsArrayList.clear();
                        variantsArrayList.addAll(priceVariants);
                    }
                    AddProductSubAddonBottomSheetFragment dialogFragment = AddProductSubAddonBottomSheetFragment.getInstance(variantsArrayList, selectedCategoryId);
                    dialogFragment.show(getChildFragmentManager(), "add price variant");
                    dialogFragment.setDialogDismissListener(new DialogDismissListener() {
                        @Override
                        public void onDialogDismiss(Object o) {
                            AddMainAddon addAddon = (AddMainAddon) o;
                            addMainAddons.add(addAddon);
                            for (AddSubAddon subAddon : addAddon.sub_addonses) {
                                if (subAddon.selected) {
                                    SubAddon subAddon1 = new SubAddon();
                                    subAddon1.subaddons_name = subAddon.subaddons_name;
                                    subAddon1.subaddons_price = subAddon.subaddons_price;
                                    selectedAddons.add(subAddon1);
                                    subAddonAdapter.notifyDataSetChanged();
                                }
                            }
                        }
                    });

//                    dialogFragment.setDialogDismissListener(o -> {
//                        AddMainAddon addAddon = (AddMainAddon) o;
//                        addMainAddons.add(addAddon);
//                        for (AddSubAddon subAddon : addAddon.sub_addonses) {
//                            if (subAddon.selected) {
//                                SubAddon subAddon1 = new SubAddon();
//                                subAddon1.subaddons_name = subAddon.subaddons_name;
//                                subAddon1.subaddons_price = subAddon.subaddons_price;
//                                selectedAddons.add(subAddon1);
//                                subAddonAdapter.notifyDataSetChanged();
//                            }
//                        }
//                    });
                }
            } else {
                ToastUtils.makeToast(getActivity(), "Please select category first");
            }
        });

        rbGroup.setOnCheckedChangeListener((group, checkedId) -> {
            switch (checkedId) {
                case R.id.rbSingle:
                    llSingleLayout.setVisibility(View.VISIBLE);
                    llMultipleLayout.setVisibility(View.GONE);
                    break;
                case R.id.rbMultiple:
                    llMultipleLayout.setVisibility(View.VISIBLE);
                    llSingleLayout.setVisibility(View.GONE);
                    break;
            }
        });

        rbAddonGroup.setOnCheckedChangeListener((group, checkedId) -> {
            switch (checkedId) {
                case R.id.rbYes:
                    llSubAddonLayout.setVisibility(View.VISIBLE);
                    break;
                case R.id.rbNo:
                    llSubAddonLayout.setVisibility(View.GONE);
                    break;
            }
        });

        btnCancel.setOnClickListener(view -> {
            dismiss();
        });

        btnConfirm.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (isValid()) {
                    Product product = new Product();

                    product.restaurant_id = myApp.getMyPreferences().getMerchantBusinesses().restaurant_id;
                    product.menu_name = etProduct.getText().toString();
                    product.short_name = etShortName.getText().toString();
                    product.menu_description = etProductDesc.getText().toString();
                    product.category_id = String.valueOf(selectedCategoryId);
                    product.sortorder = etSortOrder.getText().toString();
                    if (tvSwitchStatus.getText().toString().equalsIgnoreCase("active")) {
                        product.status = "1";
                    } else {
                        product.status = "0";
                    }
                    if (llSingleLayout.getVisibility() == View.VISIBLE)
                        product.price_option = "single";
                    else if (llMultipleLayout.getVisibility() == View.VISIBLE)
                        product.price_option = "multiple";
                    product.product_percentage = etProductDiscount.getText().toString();
                    if (llSubAddonLayout.getVisibility() == View.VISIBLE)
                        product.menu_addon = "yes";
                    else
                        product.menu_addon = "no";
                    for (int i = 0; i < allergensArrayList.size(); i++) {
                        if (allergensArrayList.get(i).name.equalsIgnoreCase("Vegetarians")) {
                            product.vegetarian = allergensArrayList.get(i).isSelected ? "1" : "0";
                        } else if (allergensArrayList.get(i).name.equalsIgnoreCase("Contains nut")) {
                            product.contains_nuts = allergensArrayList.get(i).isSelected ? "1" : "0";
                        } else if (allergensArrayList.get(i).name.equalsIgnoreCase("Milk")) {
                            product.milk = allergensArrayList.get(i).isSelected ? "1" : "0";
                        } else if (allergensArrayList.get(i).name.equalsIgnoreCase("Mustard")) {
                            product.mustard = allergensArrayList.get(i).isSelected ? "1" : "0";
                        } else if (allergensArrayList.get(i).name.equalsIgnoreCase("Eggs")) {
                            product.eggs = allergensArrayList.get(i).isSelected ? "1" : "0";
                        } else if (allergensArrayList.get(i).name.equalsIgnoreCase("Fish")) {
                            product.fish = allergensArrayList.get(i).isSelected ? "1" : "0";
                        } else if (allergensArrayList.get(i).name.equalsIgnoreCase("Gulten Free")) {
                            product.gluten_free = allergensArrayList.get(i).isSelected ? "1" : "0";
                        } else if (allergensArrayList.get(i).name.equalsIgnoreCase("Vegan")) {
                            product.vegan = allergensArrayList.get(i).isSelected ? "1" : "0";
                        } else if (allergensArrayList.get(i).name.equalsIgnoreCase("Crustaceans")) {
                            product.crustaceans = allergensArrayList.get(i).isSelected ? "1" : "0";
                        } else if (allergensArrayList.get(i).name.equalsIgnoreCase("Molluscs")) {
                            product.molluscs = allergensArrayList.get(i).isSelected ? "1" : "0";
                        } else if (allergensArrayList.get(i).name.equalsIgnoreCase("Sulphur dioxide")) {
                            product.sulphur_dioxide = allergensArrayList.get(i).isSelected ? "1" : "0";
                        } else if (allergensArrayList.get(i).name.equalsIgnoreCase("Sulphites")) {
                            product.sulphites = allergensArrayList.get(i).isSelected ? "1" : "0";
                        }
                    }
                    LogUtils.e("product::" + new Gson().toJson(product));
//                    LogUtils.e("variantsArrayList::" + new Gson().toJson(variantsArrayList));
                    LogUtils.e("priceVariants::" + new Gson().toJson(priceVariants));
                    LogUtils.e("addMainAddons::" + new Gson().toJson(addMainAddons));
//                    LogUtils.e("selectedAddons::" + new Gson().toJson(selectedAddons));

                    int flag = 0;
                    for (Variants variants : variantsArrayList) {
                        Variants mainDetail = new Variants();
                        mainDetail.id = variants.id;
                        mainDetail.menu_id = variants.menu_id;
                        mainDetail.sub_name = variants.sub_name;
                        mainDetail.orginal_price = variants.orginal_price;
                        for (AddMainAddon mainAddon : addMainAddons) {
                            AddAddon productMainAddon = new AddAddon();
                            productMainAddon.id = mainAddon.id;
                            productMainAddon.restaurant_id = myApp.getMyPreferences().getLoggedInRestaurant().id;
                            productMainAddon.category_id = Integer.parseInt(mainAddon.category_id);
                            productMainAddon.admin_name = mainAddon.admin_name;
                            productMainAddon.mainaddons_name = mainAddon.mainaddons_name;
                            productMainAddon.mainaddons_mini_count = mainAddon.mainaddons_mini_count;
                            productMainAddon.mainaddons_count = mainAddon.mainaddons_count;
                            productMainAddon.sortorder = mainAddon.sortorder;
                            productMainAddon.status = mainAddon.status;
//                            productMainAddon.delete_status = mainAddon.delete_status;

                            for (AddSubAddon subAddon : mainAddon.sub_addonses) {
                                AddSubAddon addSubAddon = new AddSubAddon();
                                addSubAddon.id = subAddon.id;
                                addSubAddon.mainaddons_id = subAddon.mainaddons_id;
                                addSubAddon.category_id = subAddon.category_id;
                                addSubAddon.subaddons_name = subAddon.subaddons_name;
                                addSubAddon.selected = subAddon.selected;
                                if (addSubAddon.selected) {
                                    addSubAddon.subaddons_price = mainAddon.priceList.get(flag).title;
                                }
                                productMainAddon.sub_addonses.add(addSubAddon);
                            }
                            mainDetail.main_addons.add(productMainAddon);
//                            mainDetailArrayList.add(mainDetail);
                            product.menu_details.add(mainDetail);
                            LogUtils.e("mainDetail::" + new Gson().toJson(mainDetail));
                        }
                        flag++;
                    }
                    LogUtils.e("product::" + new Gson().toJson(product));
                }
            }
        });
    }

    private boolean isSelectedItem() {
        if (rbSingle.isChecked()) {
            if (Validators.isNullOrEmpty(etProduct.getText().toString())) {
                ToastUtils.makeToast(getActivity(), "Please enter product name");
                return false;
            }
            if (Validators.isNullOrEmpty(etProductPrice.getText().toString())) {
                ToastUtils.makeToast(getActivity(), "Please enter product price");
                return false;
            }
            if (!Validators.isNullOrEmpty(etProductPrice.getText().toString()) && Float.parseFloat(etProductPrice.getText().toString()) <= 0) {
                ToastUtils.makeToast(getActivity(), "Please enter product price");
                return false;
            }
            return true;
        }
        if (rbMultiple.isChecked() && priceVariants.size() == 0) {
            ToastUtils.makeToast(getActivity(), "Please add price variants");
            return false;
        }
        return true;
    }

    private boolean isValid() {
        etProduct.setError(null);
        etShortName.setError(null);

        if (Validators.isNullOrEmpty(etProduct.getText().toString())) {
            etProduct.setError("Please enter product name");
            return false;
        }

        if (Validators.isNullOrEmpty(etShortName.getText().toString())) {
            etShortName.setError("Please enter product short name");
            return false;
        }

        if (Validators.isNullOrEmpty(etProductDesc.getText().toString())) {
            etProductDesc.setError("Please enter product description");
            return false;
        }

        if (Validators.isNullOrEmpty(etCategory.getText().toString())) {
            etCategory.setError("Please enter category name");
            return false;
        }

        if (Validators.isNullOrEmpty(etSortOrder.getText().toString())) {
            etSortOrder.setError("Please enter sort order");
            return false;
        }

        if (rbSingle.isChecked() && Validators.isNullOrEmpty(etProductPrice.getText().toString()) || etProductPrice.getText().toString().equalsIgnoreCase("0.00")) {
            etProductPrice.setError("Please enter product price");
            return false;
        }

        if (rbMultiple.isChecked() && priceVariants.size() == 0) {
            ToastUtils.makeToast(getActivity(), "Please add product price");
            return false;
        }

//        if (Validators.isNullOrEmpty(etProductPrice.getText().toString()) || etProductPrice.getText().toString().equalsIgnoreCase("0.00")) {
//            etProductPrice.setError("Please enter product price");
//            return false;
//        }

        return true;

    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        llAddPriceVariant = view.findViewById(R.id.ll_PriceVariants);
        llMultipleLayout = view.findViewById(R.id.llMultipleLayout);
        llSingleLayout = view.findViewById(R.id.llSingleLayout);
        llSubAddonLayout = view.findViewById(R.id.llSubAddonLayout);
        rvPriceVariant = view.findViewById(R.id.rvPriceVariant);
        rvAllergies = view.findViewById(R.id.rvAllergens);
        rvSubAddon = view.findViewById(R.id.rvSubAddon);
        llAddSubAddon = view.findViewById(R.id.llSubAddon);
        tvAddImage = view.findViewById(R.id.tvAddImage);
        cvNoImage = view.findViewById(R.id.cvNoImage);
        etCategory = view.findViewById(R.id.etCategory);
        btnConfirm = view.findViewById(R.id.btnConfirm);
        btnCancel = view.findViewById(R.id.btnCancel);
        rbNo = view.findViewById(R.id.rbNo);
        rbYes = view.findViewById(R.id.rbYes);
        rbSingle = view.findViewById(R.id.rbSingle);
        rbMultiple = view.findViewById(R.id.rbMultiple);
        rbGroup = view.findViewById(R.id.rbGroup);
        rbAddonGroup = view.findViewById(R.id.rbAddonGroup);
        switchOrder = view.findViewById(R.id.switchOrders);
        tvSwitchStatus = view.findViewById(R.id.tvSwitchStatus);
        etProductPrice = view.findViewById(R.id.etProductPrice);
        etProductDiscount = view.findViewById(R.id.etProductDiscount);
        etSortOrder = view.findViewById(R.id.etSortOrder);
        etProduct = view.findViewById(R.id.etProduct);
        etShortName = view.findViewById(R.id.etShortName);
        etProductDesc = view.findViewById(R.id.etDescription);
        ivImage = view.findViewById(R.id.ivImage);

        etProductDiscount.addTextChangedListener(new MoneyTextWatcher(etProductDiscount));
        etCategory.setOnItemClickListener((adapterView, view1, i, l) -> {
            int indext = categories.indexOf(adapterView.getItemAtPosition(i));
            if (indext != -1) {
                selectedCategoryId = Integer.parseInt(categories.get(indext).id);
                LogUtils.e("Category name is::" + categories.get(indext).category_name + "and id::" + selectedCategoryId);
            }

        });

        priceVarientAdapter = new PriceVarientListAdapter(priceVariants, new RecyclerViewItemClickListener() {
            @Override
            public void onItemClick(int position, Object data) {
//editclick
                if (data instanceof Variants) {
                    Variants variant = (Variants) data;
                    AddPriceVariantDialogFragment dialogFragment = AddPriceVariantDialogFragment.getInstance(variant, position, true);
                    dialogFragment.show(getChildFragmentManager(), "add price variant");
                    dialogFragment.setDialogDismissListener(new DialogDismissListener() {
                        @Override
                        public void onDialogDismiss(Object o) {
                            Variants header = (Variants) o;
                            LogUtils.e("Price Variant Data::" + new Gson().toJson(header));
                            if (priceVariants.get(position).position == position) {
                                priceVariants.remove(position);
                                priceVariants.add(header);
                            }
                            priceVarientAdapter.notifyDataSetChanged();
                        }
                    });

//                    dialogFragment.setDialogDismissListener(o -> {
//                        Variants header = (Variants) o;
//                        LogUtils.e("Price Variant Data::" + new Gson().toJson(header));
//                        if (priceVariants.get(position).position == position) {
//                            priceVariants.remove(position);
//                            priceVariants.add(header);
//                        }
//                        priceVarientAdapter.notifyDataSetChanged();
//                    });
                }
            }
        }, new RecyclerViewItemClickListener() {
            @Override
            public void onItemClick(int position, Object data) {
                //required click
            }
        });

        rvPriceVariant.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvPriceVariant.setAdapter(priceVarientAdapter);


        subAddonAdapter = new SubAddonListAdapter(selectedAddons, new RecyclerViewItemClickListener() {
            @Override
            public void onItemClick(int position, Object data) {

            }
        }, new RecyclerViewItemClickListener() {
            @Override
            public void onItemClick(int position, Object data) {

            }
        });

        rvSubAddon.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvSubAddon.setAdapter(subAddonAdapter);


        allergiesitemAdapter = new AllergiesitemAdapter(getActivity(), allergensArrayList, new RecyclerViewItemClickListener() {
            @Override
            public void onItemClick(int position, Object data) {
                Allergens allergens = (Allergens) data;
                LogUtils.e("selected allegries::" + new Gson().toJson(allergens));
            }
        });
        rvAllergies.setLayoutManager(new GridLayoutManager(getActivity(), 3));
        rvAllergies.setAdapter(allergiesitemAdapter);


        allergens = new Allergens();
        allergens.name = "Vegetarians";
        allergensArrayList.add(allergens);
        allergiesitemAdapter.notifyDataSetChanged();

        allergens = new Allergens();
        allergens.name = "Contains nut";
        allergensArrayList.add(allergens);
        allergiesitemAdapter.notifyDataSetChanged();

        allergens = new Allergens();
        allergens.name = "Milk";
        allergensArrayList.add(allergens);
        allergiesitemAdapter.notifyDataSetChanged();

        allergens = new Allergens();
        allergens.name = "Mustard";
        allergensArrayList.add(allergens);
        allergiesitemAdapter.notifyDataSetChanged();

        allergens = new Allergens();
        allergens.name = "Eggs";
        allergensArrayList.add(allergens);
        allergiesitemAdapter.notifyDataSetChanged();

        allergens = new Allergens();
        allergens.name = "Fish";
        allergensArrayList.add(allergens);
        allergiesitemAdapter.notifyDataSetChanged();

        allergens = new Allergens();
        allergens.name = "Gulten Free";
        allergensArrayList.add(allergens);
        allergiesitemAdapter.notifyDataSetChanged();

        allergens = new Allergens();
        allergens.name = "Vegan";
        allergensArrayList.add(allergens);
        allergiesitemAdapter.notifyDataSetChanged();

        allergens = new Allergens();
        allergens.name = "Crustaceans";
        allergensArrayList.add(allergens);
        allergiesitemAdapter.notifyDataSetChanged();

        allergens = new Allergens();
        allergens.name = "Molluscs";
        allergensArrayList.add(allergens);
        allergiesitemAdapter.notifyDataSetChanged();

        allergens = new Allergens();
        allergens.name = "Sulphur dioxide";
        allergensArrayList.add(allergens);
        allergiesitemAdapter.notifyDataSetChanged();

        allergens = new Allergens();
        allergens.name = "Sulphites";
        allergensArrayList.add(allergens);
        allergiesitemAdapter.notifyDataSetChanged();


    }

    private void fetchCategory() {
        try {
            LogUtils.e("restaurant_id: " + myApp.getMyPreferences().getMerchantBusinesses().restaurant_id);
            AndroidNetworking.get(ApiEndPoints.online_categories)
                    .addHeaders("restaurant_id", myApp.getMyPreferences().getMerchantBusinesses().restaurant_id)
                    .addQueryParameter("restaurant_id", myApp.getMyPreferences().getMerchantBusinesses().restaurant_id)
                    .addQueryParameter("nopaginate", "1")
                    .build()
                    .getAsObjectList(Categories.class, new ParsedRequestListener<List<Categories>>() {
                        @Override
                        public void onResponse(List<Categories> response) {
                            try {
                                categories.clear();
                                categories.addAll(response);
                                etCategory.setThreshold(1);
                                autoadapter = new CategoryAutoSuggestedAdapter(getActivity(), categories);
                                etCategory.setAdapter(autoadapter);

                                LogUtils.e("Categories::" + new Gson().toJson(response));
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }

                        @Override
                        public void onError(ANError anError) {
                            try {
                                anError.printStackTrace();
                            } catch (Exception e) {
                                e.printStackTrace();
                            }

                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    private void pickImage() {
        ImagePicker.Companion.with(this)
                .crop()                    //Crop image(Optional), Check Customization for more option
                .compress(1024)            //Final image size will be less than 1 MB(Optional)
                .maxResultSize(1080, 1080)    //Final image resolution will be less than 1080 x 1080(Optional)
                .start(CODE_IMAGE_PICKER);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, @Nullable Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        try {
            if (requestCode == CODE_IMAGE_PICKER) {
                if (data != null) {
                    File userImageFile = ImagePicker.Companion.getFile(data);
                    if (userImageFile != null && userImageFile.exists()) {
                        ivImage.setImageURI(Uri.fromFile(userImageFile));
                        cvNoImage.setVisibility(View.GONE);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, String permissions[], int[] grantResults) {
        try {
            if (grantResults.length > 0) {
                boolean StoragePermission = grantResults[0] ==
                        PackageManager.PERMISSION_GRANTED;

                if (!StoragePermission) {
                    ToastUtils.makeToast(getActivity(), "Permissions denied");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }
}