package com.ubsidi.menu.fragments;

import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatEditText;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.google.gson.Gson;
import com.ubsidi.menu.R;
import com.ubsidi.menu.adapters.CategoryAdapter;
import com.ubsidi.menu.base.BaseFragment;
import com.ubsidi.menu.common.CommonFunctions;
import com.ubsidi.menu.common.Validators;
import com.ubsidi.menu.interfaces.DialogDismissListener;
import com.ubsidi.menu.interfaces.RecyclerViewItemClickListener;
import com.ubsidi.menu.models.AddAddon;
import com.ubsidi.menu.models.Addon;
import com.ubsidi.menu.network.ApiEndPoints;
import com.ubsidi.menu.utils.LogUtils;
import com.ubsidi.menu.utils.ToastUtils;

import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

public class MenuAddonFragment extends BaseFragment {

    private LinearLayout llMainSearch, llSearch, llAdd, llData;
    private TextView tvMenu, tvCancel;
    private AppCompatImageView ivSearch;
    private AppCompatEditText etSearch;
    private SwipeRefreshLayout swipeRefreshLayout;
    private RecyclerView rvMenu;
    private ArrayList<String> menus = new ArrayList<>();
    private TextView tvEditDelete;
    private LottieAnimationView animationView;
    private CategoryAdapter categoryAdapter;
    private ArrayList<Addon> addons = new ArrayList<>();
    private ArrayList<Object> filteredList = new ArrayList<>();
    private String title, service;

    public static MenuAddonFragment getInstance(String title, String service) {
        MenuAddonFragment fragment = new MenuAddonFragment();
        Bundle args = new Bundle();
        args.putString("title", title);
        args.putString("service", service);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_menu_addon, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        setListeners();
        fetchAddons();
    }

    private void setListeners() {
        swipeRefreshLayout.setOnRefreshListener(() -> {
            fetchAddons();
        });
        ivSearch.setOnClickListener(view -> {
            llMainSearch.setVisibility(View.GONE);
            llSearch.setVisibility(View.VISIBLE);
        });

        tvCancel.setOnClickListener(view -> {
            etSearch.setText(null);
            filteredList.clear();
            filteredList.addAll(addons);
            categoryAdapter.notifyDataSetChanged();
            llMainSearch.setVisibility(View.VISIBLE);
            llSearch.setVisibility(View.GONE);
        });

        etSearch.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                filteredList.clear();
                filteredList.addAll(searchAddon());
                categoryAdapter.notifyDataSetChanged();
            }
        });
        llAdd.setOnClickListener(view -> {
            AddOrEditMenuAddonDialogFragment fragment = AddOrEditMenuAddonDialogFragment.getInstance(null);
            fragment.show(getChildFragmentManager(), "Add Category");
            fragment.setDialogDismissListener(new DialogDismissListener() {
                @Override
                public void onDialogDismiss(Object o) {
                    AddAddon addon = (AddAddon) o;
                    LogUtils.e("Add Addon ::" + new Gson().toJson(addon));
                    addOrEditAddon(addon, "Add");
                }
            }, new DialogDismissListener() {
                @Override
                public void onDialogDismiss(Object o) {

                }
            });

//            fragment.setDialogDismissListener(o -> {
//                AddAddon addon = (AddAddon) o;
//                LogUtils.e("Add Addon ::" + new Gson().toJson(addon));
//                addOrEditAddon(addon, "Add");
//            }, o -> {
//
//            });


        });
    }

    private void addOrEditAddon(AddAddon addon, String action) {
        String urls = null;
        if (action.equalsIgnoreCase("edit"))
            urls = ApiEndPoints.online_addons + addon.id;
        else
            urls = ApiEndPoints.online_addons;
        AndroidNetworking.post(urls)
                .addApplicationJsonBody(addon)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        try {
                            if (getActivity() != null) {
                                getActivity().runOnUiThread(() -> {
                                    fetchAddons();
                                    categoryAdapter.notifyDataSetChanged();
                                });
                                LogUtils.e("add addons responce::" + new Gson().toJson(response));
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        }

                    }

                    @Override
                    public void onError(ANError anError) {
                        try {
                            anError.printStackTrace();
                            if (getActivity() != null) {
                                getActivity().runOnUiThread(() -> {
                                    swipeRefreshLayout.setRefreshing(false);
                                    CommonFunctions.hideKeyboard(getActivity(), etSearch);
                                    LogUtils.e("ERROR IN ADDING OR EDIT ADDON::" + new Gson().toJson(anError.getErrorBody()));
                                    fetchAddons();
                                });
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        }

                    }
                });
    }

    private ArrayList<Addon> searchAddon() {
        if (etSearch.getText().toString().isEmpty()) {
            return addons;
        }
        ArrayList<Addon> searchedRest = new ArrayList<>();
        for (Addon r : addons) {
            if (r.mainaddons_name != null && r.mainaddons_name.toLowerCase().contains(etSearch.getText().toString().toLowerCase())) {
                searchedRest.add(r);
            }
        }
        return searchedRest;

    }

    private void fetchAddons() {
        try {
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    llData.setVisibility(View.VISIBLE);
                    swipeRefreshLayout.setRefreshing(true);
                    animationView.setVisibility(View.GONE);
                });
            }
            AndroidNetworking.get(ApiEndPoints.online_addons)
                    .addHeaders("restaurant_id", myApp.getMyPreferences().getLoggedInRestaurant().id)
                    .addQueryParameter("nopaginate", "1")
                    .build()
                    .getAsObjectList(Addon.class, new ParsedRequestListener<List<Addon>>() {
                        @Override
                        public void onResponse(List<Addon> response) {
                            try {
                                if (getActivity() != null) {
                                    getActivity().runOnUiThread(() -> {
                                        llData.setVisibility(View.VISIBLE);
                                        swipeRefreshLayout.setRefreshing(false);
                                        animationView.setVisibility(View.GONE);
                                    });

                                    addons.clear();
                                    addons.addAll(response);
                                    filteredList.clear();
                                    filteredList.addAll(response);
                                    categoryAdapter.notifyDataSetChanged();

                                }
                            } catch (Exception e) {
                                e.printStackTrace();
                            }

                        }

                        @Override
                        public void onError(ANError anError) {
                            try {
                                anError.printStackTrace();
                                if (getActivity() != null) {
                                    getActivity().runOnUiThread(() -> {
                                        llData.setVisibility(View.GONE);
                                        swipeRefreshLayout.setRefreshing(false);
                                        animationView.setVisibility(View.VISIBLE);
                                    });
                                }
                            } catch (Exception e) {
                                e.printStackTrace();
                            }

                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    swipeRefreshLayout.setRefreshing(false);
                });
            }

        }

    }

    @Override
    protected void manageIntents() {
        super.manageIntents();
        if (getArguments() != null) {
            title = getArguments().getString("title");
            service = getArguments().getString("service");
        }
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);
        llMainSearch = view.findViewById(R.id.llMainSearch);
        llSearch = view.findViewById(R.id.llSearchLayout);
        tvMenu = view.findViewById(R.id.tvMenu);
        ivSearch = view.findViewById(R.id.ivSearch);
        llAdd = view.findViewById(R.id.llAdd);
        etSearch = view.findViewById(R.id.etSearch);
        tvCancel = view.findViewById(R.id.tvCancel);
        rvMenu = view.findViewById(R.id.rvCategories);
        animationView = view.findViewById(R.id.animation_view);
        llData = view.findViewById(R.id.llRootData);

        if (!Validators.isNullOrEmpty(title))
            tvMenu.setText(title);
        categoryAdapter = new CategoryAdapter(filteredList, new RecyclerViewItemClickListener() {
            @Override
            public void onItemClick(int position, Object data) {

                if (data instanceof Addon) {
                    Addon addon = (Addon) data;
                    openEditAddonDialog(addon);

                }
            }
        });
        rvMenu.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
        rvMenu.setNestedScrollingEnabled(false);
        rvMenu.setAdapter(categoryAdapter);

    }

    private void openEditAddonDialog(Addon addon) {
        AddOrEditMenuAddonDialogFragment fragment = AddOrEditMenuAddonDialogFragment.getInstance(addon);
        fragment.show(getChildFragmentManager(), "Edit Addon");
        fragment.setCancelable(false);
        fragment.setDialogDismissListener(new DialogDismissListener() {
            @Override
            public void onDialogDismiss(Object o) {
                AddAddon addon1 = (AddAddon) o;

                addOrEditAddon(addon1, "Edit");
            }
        }, new DialogDismissListener() {
            @Override
            public void onDialogDismiss(Object o) {
                Addon addon1 = (Addon) o;
                deleteItem(addon1.id);
            }
        });

//        fragment.setDialogDismissListener(o -> {
//            AddAddon addon1 = (AddAddon) o;
//
//            addOrEditAddon(addon1, "Edit");
//        }, o -> {
//            Addon addon1 = (Addon) o;
//            deleteItem(addon1.id);
//        });
    }

    private void deleteItem(String id) {
        AndroidNetworking.delete(ApiEndPoints.online_addons + id)
                .build()
                .getAsJSONObject(new JSONObjectRequestListener() {
                    @Override
                    public void onResponse(JSONObject response) {
                        try {
                            if (getActivity() != null) {
                                getActivity().runOnUiThread(() -> {
                                    fetchAddons();
                                });
                                ToastUtils.makeToast(getActivity(), "Addon Deleted");
                                LogUtils.e("delete addon responce::" + new Gson().toJson(response));
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }

                    @Override
                    public void onError(ANError anError) {
                        try {
                            anError.printStackTrace();
                            if (getActivity() != null) {
                                getActivity().runOnUiThread(() -> {
                                    swipeRefreshLayout.setRefreshing(false);
                                });
                            }
                        } catch (Exception e) {
                            e.printStackTrace();
                        }

                    }
                });

    }
}