package com.ubsidi.menu.fragments;

import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.appcompat.widget.AppCompatEditText;
import androidx.appcompat.widget.AppCompatImageView;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout;

import android.text.Editable;
import android.text.TextWatcher;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.TextView;

import com.airbnb.lottie.LottieAnimationView;
import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.ubsidi.menu.R;
import com.ubsidi.menu.adapters.CategoryAdapter;
import com.ubsidi.menu.base.BaseFragment;
import com.ubsidi.menu.common.Validators;
import com.ubsidi.menu.interfaces.DialogDismissListener;
import com.ubsidi.menu.models.Product;
import com.ubsidi.menu.network.ApiEndPoints;

import java.util.ArrayList;
import java.util.List;

public class MenuProductsFragment extends BaseFragment {

    private LinearLayout llMainSearch, llSearch, llAdd, llData;
    private TextView tvMenu, tvCancel;
    private AppCompatImageView ivSearch;
    private AppCompatEditText etSearch;
    private SwipeRefreshLayout swipeRefreshLayout;
    private RecyclerView rvMenu;
    private ArrayList<Product> products = new ArrayList<>();
    private ArrayList<Object> filteredList = new ArrayList<>();
    private CategoryAdapter productAdapter;
    private LottieAnimationView animationView;
    private LinearLayoutManager menuLayoutManager;
    private String title, service;

    public static MenuProductsFragment getInstance(String title, String service) {
        MenuProductsFragment fragment = new MenuProductsFragment();
        Bundle args = new Bundle();
        args.putString("title", title);
        args.putString("service", service);
        fragment.setArguments(args);
        return fragment;
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_menu_products, container, false);
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        setListeners();
        fetchProduct();
    }

    private void fetchProduct() {
        try {
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    llData.setVisibility(View.VISIBLE);
                    swipeRefreshLayout.setRefreshing(true);
                    animationView.setVisibility(View.GONE);
                });
            }
            AndroidNetworking.get(ApiEndPoints.online_products)
                    .addHeaders("restaurant_id", myApp.getMyPreferences().getLoggedInRestaurant().id)
                    .addQueryParameter("nopaginate", "1")
                    .build()
                    .getAsObjectList(Product.class, new ParsedRequestListener<List<Product>>() {
                        @Override
                        public void onResponse(List<Product> response) {
                            try {
                                if (getActivity() != null) {
                                    getActivity().runOnUiThread(() -> {
                                        llData.setVisibility(View.VISIBLE);
                                        swipeRefreshLayout.setRefreshing(false);
                                        animationView.setVisibility(View.GONE);
                                    });

                                    products.clear();
                                    products.addAll(response);
                                    filteredList.clear();
                                    filteredList.addAll(response);
                                    productAdapter.notifyDataSetChanged();

                                }
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }

                        @Override
                        public void onError(ANError anError) {
                            try {
                                anError.printStackTrace();
                                if (getActivity() != null) {
                                    getActivity().runOnUiThread(() -> {
                                        llData.setVisibility(View.GONE);
                                        swipeRefreshLayout.setRefreshing(false);
                                        animationView.setVisibility(View.VISIBLE);
                                    });
                                }
                            } catch (Exception e) {
                                e.printStackTrace();
                            }

                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
            if (getActivity() != null) {
                getActivity().runOnUiThread(() -> {
                    swipeRefreshLayout.setRefreshing(false);
                });
            }

        }
    }

    private void setListeners() {
        ivSearch.setOnClickListener(view -> {
            llMainSearch.setVisibility(View.GONE);
            llSearch.setVisibility(View.VISIBLE);
        });

        tvCancel.setOnClickListener(view -> {
            llMainSearch.setVisibility(View.VISIBLE);
            llSearch.setVisibility(View.GONE);
        });

        llAdd.setOnClickListener(view -> {
            AddProductBottomSheetFragment fragment = AddProductBottomSheetFragment.getInstance(null);
            fragment.show(getChildFragmentManager(), "add product");
            fragment.setCancelable(false);
            fragment.setDialogDismissListener(new DialogDismissListener() {
                @Override
                public void onDialogDismiss(Object o) {

                }
            });
        });
        tvCancel.setOnClickListener(view -> {
            etSearch.setText(null);
            filteredList.clear();
            filteredList.addAll(products);
            productAdapter.notifyDataSetChanged();
            llMainSearch.setVisibility(View.VISIBLE);
            llSearch.setVisibility(View.GONE);
        });

        etSearch.addTextChangedListener(new TextWatcher() {
            @Override
            public void beforeTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void onTextChanged(CharSequence charSequence, int i, int i1, int i2) {

            }

            @Override
            public void afterTextChanged(Editable editable) {
                filteredList.clear();
                filteredList.addAll(searchCategory());
                productAdapter.notifyDataSetChanged();
            }
        });

    }

    @Override
    protected void manageIntents() {
        super.manageIntents();
        if (getArguments() != null) {
            title = getArguments().getString("title");
            service = getArguments().getString("service");
        }
    }


    @Override
    protected void initViews(View view) {
        super.initViews(view);
        llMainSearch = view.findViewById(R.id.llMainSearch);
        llSearch = view.findViewById(R.id.llSearchLayout);
        tvMenu = view.findViewById(R.id.tvMenu);
        ivSearch = view.findViewById(R.id.ivSearch);
        llAdd = view.findViewById(R.id.llAdd);
        etSearch = view.findViewById(R.id.etSearch);
        tvCancel = view.findViewById(R.id.tvCancel);
        rvMenu = view.findViewById(R.id.rvCategories);
        swipeRefreshLayout = view.findViewById(R.id.swipeRefreshLayout);
        animationView = view.findViewById(R.id.animation_view);
        llData = view.findViewById(R.id.llRootData);

        if (!Validators.isNullOrEmpty(title))
            tvMenu.setText(title);
        productAdapter = new CategoryAdapter(filteredList, (position, data) -> {

            if (data instanceof Product) {
                Product categories = (Product) data;
//                    openEditCategoryDialog(categories);

            }
        });
        menuLayoutManager = new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false);
        rvMenu.setLayoutManager(menuLayoutManager);
        rvMenu.setNestedScrollingEnabled(false);
        rvMenu.setAdapter(productAdapter);


    }

    private ArrayList<Product> searchCategory() {
        if (etSearch.getText().toString().isEmpty()) {
            return products;
        }
        ArrayList<Product> searchedRest = new ArrayList<>();
        for (Product r : products) {
            if (r.menu_name != null && r.menu_name.toLowerCase().contains(etSearch.getText().toString().toLowerCase())) {
                searchedRest.add(r);
            }
        }
        return searchedRest;

    }
}