package com.ubsidi.menu.fragments;

import android.content.Intent;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;
import androidx.fragment.app.Fragment;
import androidx.recyclerview.widget.LinearLayoutManager;
import androidx.recyclerview.widget.RecyclerView;

import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.TextView;
import android.widget.Toast;

import com.androidnetworking.AndroidNetworking;
import com.androidnetworking.error.ANError;
import com.androidnetworking.interfaces.JSONArrayRequestListener;
import com.androidnetworking.interfaces.JSONObjectRequestListener;
import com.androidnetworking.interfaces.ParsedRequestListener;
import com.google.android.material.button.MaterialButton;
import com.google.gson.Gson;
import com.ubsidi.menu.R;
import com.ubsidi.menu.activities.RegisterDeviceActivity;
import com.ubsidi.menu.activities.SplashActivity;
import com.ubsidi.menu.activities.TransportActivity;
import com.ubsidi.menu.adapters.MerchantBusinessSelectionAdapter;
import com.ubsidi.menu.base.BaseFragment;
import com.ubsidi.menu.common.Validators;
import com.ubsidi.menu.models.ApiError;
import com.ubsidi.menu.models.CardBrandCommission;
import com.ubsidi.menu.models.CountryCommission;
import com.ubsidi.menu.models.DeviceRegistration;
import com.ubsidi.menu.models.MerchantBusinesses;
import com.ubsidi.menu.models.RestaurantUser;
import com.ubsidi.menu.network.ApiEndPoints;
import com.ubsidi.menu.utils.LogUtils;
import com.ubsidi.menu.utils.ToastUtils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

public class MerchantBusinessSelectionFragment extends BaseFragment {

    public static MerchantBusinessSelectionFragment getInstance() {
        return new MerchantBusinessSelectionFragment();
    }

    TextView tvSelect, tvChange;
    private RecyclerView rvBusinesses;
    private MerchantBusinessSelectionAdapter businessAdapter;
    private ArrayList<MerchantBusinesses> merchantBusinesses = new ArrayList<>();
    private MaterialButton btnDone;
    private MerchantBusinesses selectedBusinesse;

    @Override
    protected void manageIntents() {
        super.manageIntents();
    }

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container, Bundle savedInstanceState) {
        return inflater.inflate(R.layout.fragment_merchant_business_selection, container, false);
    }

    @Override
    protected void initViews(View view) {
        super.initViews(view);
        try {
            fetchBusinesses();
            rvBusinesses = view.findViewById(R.id.rvRestaurants);
            btnDone = view.findViewById(R.id.btnDone);
            businessAdapter = new MerchantBusinessSelectionAdapter(merchantBusinesses, (position, data) -> {
                if (data instanceof MerchantBusinesses) {
                    btnDone.setEnabled(true);
                    selectedBusinesse = (MerchantBusinesses) data;
                }
            });
            rvBusinesses.setLayoutManager(new LinearLayoutManager(getActivity(), RecyclerView.VERTICAL, false));
            rvBusinesses.setAdapter(businessAdapter);
            if (myApp.getMyPreferences().getMerchantBusinesses() != null) {
                selectedBusinesse = myApp.getMyPreferences().getMerchantBusinesses();
                businessAdapter.selectedMerchantBusinessId = selectedBusinesse.id;
                businessAdapter.notifyDataSetChanged();
                btnDone.setEnabled(true);
            }

            tvSelect = view.findViewById(R.id.tvSelect);
            tvChange = view.findViewById(R.id.tvChange);

//            if (Validators.isNullOrEmpty(myApp.getMyPreferences().getLoggedInRestaurant().restaurant_name)) {
//                tvSelect.setVisibility(View.VISIBLE);
//                tvChange.setVisibility(View.GONE);
//            } else {
//                tvSelect.setVisibility(View.GONE);
//                tvChange.setVisibility(View.VISIBLE);
//            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void onViewCreated(@NonNull View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        try {
            btnDone.setOnClickListener(view1 -> {
                if (selectedBusinesse == null) {
                    ToastUtils.makeToast(getActivity(), "Please select restaurant first");
                } else {
                    myApp.getMyPreferences().saveMerchantBusinesses(selectedBusinesse);
                    myApp.getMyPreferences().saveBusinessId(selectedBusinesse.id);
                    getFcmToken();
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void fetchBusinesses() {
        try {
            progressDialog.show();
            AndroidNetworking.get(ApiEndPoints.MERCHANT_BUSINESSES)
                    .addQueryParameter("request_for", "admin")
                    .addQueryParameter("nopaginate", "1")
                    .build()
                    .getAsObjectList(MerchantBusinesses.class, new ParsedRequestListener<List<MerchantBusinesses>>() {

                        @Override
                        public void onResponse(List<MerchantBusinesses> response) {
                            try {
                                merchantBusinesses.addAll(response);
                                businessAdapter.notifyDataSetChanged();
                                progressDialog.dismiss();
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }

                        @Override
                        public void onError(ANError anError) {
                            try {
                                progressDialog.dismiss();
                                if (anError.getErrorCode() == 400) {
                                    ApiError apiError = anError.getErrorAsObject(ApiError.class);
                                    ToastUtils.makeSnackToast(getActivity(), apiError.getMessage());
                                }
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void getFcmToken() {
        saveMerchantDeviceId();
        /*try {
            FirebaseMessaging.getInstance().getToken()
                    .addOnCompleteListener(task -> {
                        if (!task.isSuccessful()) {
                            Log.w("TAG", "Fetching FCM registration token failed", task.getException());
                            Toast.makeText(getActivity(), "Failed to generate FCM token, Please try again later.", Toast.LENGTH_SHORT).show();
                            return;
                        }
                        // Get new FCM registration token
                        String token = task.getResult();

                        if (!token.equalsIgnoreCase("")) {
                            Log.e("Instance"," ; "+token);
                            MyApp.getInstance().getMyPreferences().setUserFCMToken(token);
                            if (!MyApp.getInstance().getMyPreferences().getUserFCMToken().equalsIgnoreCase("")) {
                                saveMerchantDeviceId();
                            }
                        } else {
                            Toast.makeText(getActivity(), "Fcm token not found.", Toast.LENGTH_SHORT).show();
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }*/
    }

    private void saveMerchantDeviceId() {
        try {
            if (getActivity() != null)
                getActivity().runOnUiThread(() -> progressDialog.show());

            AndroidNetworking.post(ApiEndPoints.MERCHANT_DEVICE_REGISTRATION)
                    .addPathParameter("business_id", myApp.getMyPreferences().getBusinessId())
                    .addBodyParameter("business_id", myApp.getMyPreferences().getBusinessId())
                    .addBodyParameter("device_id", myApp.getMyPreferences().getAndroidDeviceId())
                    .addBodyParameter("device_type", "Android")
                    .addBodyParameter("service_type", "Merchant")
                    .addBodyParameter("device_token", myApp.getMyPreferences().getUserFCMToken())
                    .build()
                    .getAsObject(DeviceRegistration.class, new ParsedRequestListener<DeviceRegistration>() {
                        @Override
                        public void onResponse(DeviceRegistration response) {
                            try {
                                if (getActivity() != null)
                                    getActivity().runOnUiThread(() -> progressDialog.dismiss());

                                myApp.getMyPreferences().saveDeviceRegistration(response);

                                if (selectedBusinesse.online_service && !Validators.isNullOrEmpty(selectedBusinesse.restaurant_id)) {
                                    LogUtils.e("In If");
                                    fetchRestaurantDetails(selectedBusinesse.restaurant_id);
                                } else {
                                    LogUtils.e("In Else");
                                    continueNextScreen();
                                }
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }

                        @Override
                        public void onError(ANError anError) {
                            if (getActivity() != null)
                                getActivity().runOnUiThread(() -> progressDialog.dismiss());
                            anError.printStackTrace();
                            if (anError.getErrorCode() == 400) {
                                ApiError apiError = anError.getErrorAsObject(ApiError.class);
                                ToastUtils.makeSnackToast(getActivity(), apiError.getMessage());
                            }
                            if (selectedBusinesse.online_service && !Validators.isNullOrEmpty(selectedBusinesse.restaurant_id)) {
                                fetchRestaurantDetails(selectedBusinesse.restaurant_id);
                            } else {
                                continueNextScreen();
                            }
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void fetchCountryCommission() {
        try {
            AndroidNetworking.get(ApiEndPoints.businesses + "/" + myApp.getMyPreferences().getBusinessId() + "/card-brand-commissions/")
                    .addQueryParameter("nopaginate", "1")
                    .build()
                    .getAsJSONArray(new JSONArrayRequestListener() {
                        @Override
                        public void onResponse(JSONArray response) {
                            try {
                                progressDialog.dismiss();
                                ArrayList<CardBrandCommission> list = new ArrayList<>();
                                try {
                                    for (int i = 0; i < response.length(); i++) {
                                        JSONObject jsonObject = response.getJSONObject(i);
                                        CardBrandCommission cardBrandCommission = new CardBrandCommission();
                                        cardBrandCommission.id = jsonObject.getInt("id");
                                        cardBrandCommission.card_brand_id = jsonObject.getInt("card_brand_id");
                                        cardBrandCommission.business_id = jsonObject.getInt("business_id");
                                        cardBrandCommission.card_brand_id = jsonObject.getInt("card_brand_id");
                                        cardBrandCommission.stripe_percentage = jsonObject.getInt("stripe_percentage");
                                        cardBrandCommission.stripe_fixed = jsonObject.getInt("stripe_fixed");
                                        cardBrandCommission.other = jsonObject.getBoolean("other");
                                        cardBrandCommission.created_at = jsonObject.getString("created_at");
                                        JSONObject brand = jsonObject.getJSONObject("card_brand");
                                        cardBrandCommission.brand_name = brand.getString("name");
                                        list.add(cardBrandCommission);
                                    }
                                } catch (JSONException e) {
                                    e.printStackTrace();
                                }
                                myApp.appDatabase.commissionDao().nukeCardBrandCommission();
                                myApp.appDatabase.commissionDao().insertCardBrands(list);
                                continueNextScreen();
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }

                        @Override
                        public void onError(ANError anError) {
                            try {
                                anError.printStackTrace();
                                progressDialog.dismiss();
                                continueNextScreen();
                            } catch (Exception e) {
                                e.printStackTrace();
                            }

                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
            progressDialog.dismiss();
            continueNextScreen();
        }
    }

    private void fetchCountryBrandCommission() {
        try {
            AndroidNetworking.get(ApiEndPoints.businesses + "/" + myApp.getMyPreferences().getBusinessId() + "/country-commissions/")
                    .addQueryParameter("nopaginate", "1")
                    .build()
                    .getAsJSONArray(new JSONArrayRequestListener() {
                        @Override
                        public void onResponse(JSONArray response) {
                            try {
                                myApp.appDatabase.commissionDao().nukeCountryCommission();
                                ArrayList<CountryCommission> list = new ArrayList<>();
                                try {
                                    for (int i = 0; i < response.length(); i++) {
                                        JSONObject jsonObject = response.getJSONObject(i);
                                        CountryCommission cardBrandCommission = new CountryCommission();
                                        cardBrandCommission.id = jsonObject.getInt("id");
                                        cardBrandCommission.paypal_percentage = jsonObject.getInt("paypal_percentage");
                                        cardBrandCommission.paypal_fixed = jsonObject.getInt("paypal_fixed");
                                        cardBrandCommission.stripe_percentage = jsonObject.getInt("stripe_percentage");
                                        cardBrandCommission.stripe_fixed = jsonObject.getInt("stripe_fixed");
                                        cardBrandCommission.business_id = jsonObject.getInt("business_id");
                                        cardBrandCommission.other = jsonObject.getBoolean("other");
                                        cardBrandCommission.created_at = jsonObject.getString("created_at");
                                        if (!cardBrandCommission.other) {
                                            cardBrandCommission.country_id = jsonObject.getInt("country_id");
                                            JSONObject brand = jsonObject.getJSONObject("country");
                                            cardBrandCommission.name = brand.getString("name");
                                            cardBrandCommission.iso = brand.getString("iso");
                                            cardBrandCommission.currency_code = brand.getString("currency_code");
                                        } else {
                                            cardBrandCommission.country_id = 0;
                                            cardBrandCommission.name = "";
                                            cardBrandCommission.iso = "";
                                            cardBrandCommission.currency_code = "";
                                        }
                                        list.add(cardBrandCommission);
                                    }
                                } catch (JSONException e) {
                                    e.printStackTrace();
                                }
                                myApp.appDatabase.commissionDao().insertCountryCommissions(list);
                                fetchCountryCommission();
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                        }

                        @Override
                        public void onError(ANError anError) {
                            try {
                                progressDialog.dismiss();
                                fetchCountryCommission();
                                anError.printStackTrace();
                            } catch (Exception e) {
                                e.printStackTrace();
                            }

                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
            fetchCountryCommission();
            progressDialog.dismiss();
        }
    }

    private void fetchRestaurantDetails(String resId) {
        try {
            if (getActivity() != null)
                getActivity().runOnUiThread(() -> progressDialog.show());
            AndroidNetworking.get(ApiEndPoints.restaurants + resId)
                    .build()
                    .getAsJSONObject(new JSONObjectRequestListener() {
                        @Override
                        public void onResponse(JSONObject response) {
                            try {
                                myApp.getMyPreferences().saveLoggedInRestaurant(new Gson().fromJson(response.toString(), RestaurantUser.class));
                                myApp.getMyPreferences().saveSiteSettings(myApp.getMyPreferences().getLoggedInRestaurant().site_setting);
                            } catch (Exception e) {
                                e.printStackTrace();
                            }
                            updateRestaurantToken(resId);
                        }

                        @Override
                        public void onError(ANError anError) {
                            anError.printStackTrace();
                            continueNextScreen();
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void updateRestaurantToken(String resId) {
        try {
            if (getActivity() != null)
                getActivity().runOnUiThread(() -> progressDialog.show());

            HashMap<String, String> params = new HashMap<>();
            params.put("device_id", myApp.getMyPreferences().getUserFCMToken());
            AndroidNetworking.post(ApiEndPoints.restaurants + resId)
                    .addQueryParameter(params)
                    .build()
                    .getAsJSONObject(new JSONObjectRequestListener() {
                        @Override
                        public void onResponse(JSONObject response) {
                            fetchCountryBrandCommission();
                        }

                        @Override
                        public void onError(ANError anError) {
                            anError.printStackTrace();
                            progressDialog.dismiss();
                            continueNextScreen();
                        }
                    });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void continueNextScreen() {
        try {
//            if (myApp.getMyPreferences().getDeviceRegistration() != null && myApp.getMyPreferences().getDeviceRegistration().status != null
//                    && !myApp.getMyPreferences().getDeviceRegistration().status.equalsIgnoreCase("approved")) {
//                Intent intent = new Intent(getActivity(), RegisterDeviceActivity.class);
//                startActivity(intent);
//            } else {
            if (getActivity() != null)
                getActivity().runOnUiThread(() -> progressDialog.dismiss());
//                if (myApp.getMyPreferences().getDefaultPrinter() == null) {
//                    Intent intent = new Intent(getActivity(), TransportActivity.class).putExtra("destination", "printer_selection");
//                    startActivity(intent);
//                } else {
            Intent intent = new Intent(getActivity(), SplashActivity.class);
            startActivity(intent);
            if (getActivity() != null)
                getActivity().finish();
//                }
//            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}